#include "stdafx.h"
#include "cphidgetspatial.h"
#include "cusb.h"
#include "math.h"
#include "csocket.h"
#include "cthread.h"

// === Internal Functions === //

//clearVars - sets all device variables to unknown state
CPHIDGETCLEARVARS(Spatial)
	int i = 0;

	//TODO: finish initializations
	phid->accelerationMax = PUNI_DBL;
	phid->accelerationMin = PUNI_DBL;

	for (i = 0; i<SPATIAL_MAX_ACCELAXES; i++)
	{
		phid->accelAxis[i] = PUNI_DBL;
	}
	return EPHIDGET_OK;
}

//initAfterOpen - sets up the initial state of an object, reading in packets from the device if needed
//				  used during attach initialization - on every attach
CPHIDGETINIT(Spatial)
	int i = 0;

	TESTPTR(phid);

	//Setup max/min values
	switch(phid->phid.deviceIDSpec)
	{
		case PHIDID_ACCEL_3AXIS_GYRO_1AXIS:
			if (phid->phid.deviceVersion < 200)
			{
				phid->accelerationMax = 12.1;
				phid->accelerationMin = -12.1;
				phid->interruptRate = 8;
				phid->dataRateMin = SPATIAL_MIN_DATA_RATE;
				phid->dataRate = phid->interruptRate;
				phid->dataRateMax = 2; //actual data rate
				phid->angularRateMax = 300.1;
				phid->angularRateMin = -300.1;
				phid->magneticFieldMax = 0;
				phid->magneticFieldMin = 0;
			}
			else
				return EPHIDGET_UNEXPECTED;
			break;
		default:
			return EPHIDGET_UNEXPECTED;
	}

	//initialize triggers, set data arrays to unknown
	for (i = 0; i<phid->phid.attr.spatial.numAccelAxes; i++)
	{
		phid->accelAxis[i] = PUNK_DBL;
	}
	for (i = 0; i<phid->phid.attr.spatial.numGyroAxes; i++)
	{
		phid->gyroAxis[i] = PUNK_DBL;
		phid->gryoCorrection[i] = 0;
	}
	for (i = 0; i<phid->phid.attr.spatial.numCompassAxes; i++)
	{
		phid->compassAxis[i] = PUNK_DBL;
	}
	phid->bufferReadPtr = 0;
	phid->bufferWritePtr = 0;
	phid->timestamp.seconds = 0;
	phid->timestamp.microseconds = 0;
	phid->lastEventTime.seconds = 0;
	phid->lastEventTime.microseconds = 0;
	phid->latestDataTime.seconds = 0;
	phid->latestDataTime.microseconds = 0;

	phid->lastTimeCounterValid = PFALSE;
	phid->doZeroGyro = PFALSE;

	//issue one read
	//this should fill in the data because the dataRate is the interrupt rate
	CPhidget_read((CPhidgetHandle)phid);

	return EPHIDGET_OK;
}

//dataInput - parses device packets
CPHIDGETDATA(Spatial)
	int i = 0, j = 0, count = 0, dataRate = phid->dataRate;
	unsigned char doneGyroZero = PFALSE;
	double accelAvg[SPATIAL_MAX_ACCELAXES], angularRateAvg[SPATIAL_MAX_ACCELAXES], magneticFieldAvg[SPATIAL_MAX_ACCELAXES];
	CPhidgetSpatial_SpatialEventDataHandle *eventData;
	
	ZEROMEM(accelAvg, sizeof(accelAvg));
	ZEROMEM(angularRateAvg, sizeof(angularRateAvg));
	ZEROMEM(magneticFieldAvg, sizeof(magneticFieldAvg));

	if (length<0) return EPHIDGET_INVALIDARG;
	TESTPTR(phid);
	TESTPTR(buffer);

	//Parse device packets - store data locally
	switch(phidG->deviceIDSpec)
	{
		case PHIDID_ACCEL_3AXIS_GYRO_1AXIS:
			if (phidG->deviceVersion < 200)
			{
				int data;
				int time;
				
				count = buffer[0] / 4;
				if(count == 0)
					goto done;

				//this timestamp is for the latest data
				time = ((unsigned short)buffer[1]<<8) + (unsigned short)buffer[2];
				if(phid->lastTimeCounterValid)
				{
					//0-255 ms
					int timechange = (unsigned short)((unsigned short)time - (unsigned short)phid->lastTimeCounter);
					timechange *= 1000; //us

					phid->timestamp.seconds = phid->timestamp.seconds + (phid->timestamp.microseconds + timechange) / 1000000;
					phid->timestamp.microseconds = (phid->timestamp.microseconds + timechange) % 1000000;
				}
				else
				{
					phid->lastTimeCounterValid = PTRUE;
				}
				phid->lastTimeCounter = time;

				//add data to data buffer
				for(i=0;i<count;i++)
				{
					data = ((unsigned short)buffer[3 + i * 8]<<8) + (unsigned short)buffer[4 + i * 8];
					phid->dataBuffer[phid->bufferWritePtr].angularRate[0] = round_double((((double)data-0x7fff) / 65.535), 4);
					
					data = ((unsigned short)buffer[5 + i * 8]<<8) + (unsigned short)buffer[6 + i * 8];
					phid->dataBuffer[phid->bufferWritePtr].acceleration[2] = round_double((((double)data-0x7fff) / 1660.0), 4);
					data = ((unsigned short)buffer[7 + i * 8]<<8) + (unsigned short)buffer[8 + i * 8];
					phid->dataBuffer[phid->bufferWritePtr].acceleration[1] = round_double((((double)data-0x7fff) / 1660.0), 4);
					data = ((unsigned short)buffer[9 + i * 8]<<8) + (unsigned short)buffer[10 + i * 8];
					phid->dataBuffer[phid->bufferWritePtr].acceleration[0] = round_double((((double)data-0x7fff) / 1660.0), 4);

					phid->latestDataTime.seconds = phid->timestamp.seconds + (phid->timestamp.microseconds + (i + 1) * phid->dataRateMax * 1000) / 1000000;
					phid->latestDataTime.microseconds = (phid->timestamp.microseconds + (i + 1) * phid->dataRateMax * 1000) % 1000000;

					phid->dataBuffer[phid->bufferWritePtr].timestamp = phid->latestDataTime;

					phid->bufferWritePtr++;
					if(phid->bufferWritePtr >= SPATIAL_DATA_BUFFER_SIZE)
						phid->bufferWritePtr = 0;
				}
			}
			else
				return EPHIDGET_UNEXPECTED;
			break;
		default:
			return EPHIDGET_UNEXPECTED;
	}

	if(phid->doZeroGyro)
	{
		//done
		if(timestampdiff(phid->latestDataTime, phid->dataBuffer[phid->gyroZeroReadPtr].timestamp) >= SPATIAL_ZERO_GYRO_TIME)
		{
			double gryoCorrectionTemp[SPATIAL_MAX_GYROAXES] = {0,0,0};
			int gryoCorrectionCount = 0;

			while(phid->gyroZeroReadPtr != phid->bufferWritePtr)
			{
				for (i = 0; i<phid->phid.attr.spatial.numGyroAxes; i++)
				{
					gryoCorrectionTemp[i] += phid->dataBuffer[phid->gyroZeroReadPtr].angularRate[i];
				}

				phid->gyroZeroReadPtr++;
				if(phid->gyroZeroReadPtr >= SPATIAL_DATA_BUFFER_SIZE)
					phid->gyroZeroReadPtr = 0;

				gryoCorrectionCount++;
			}
			
			for (i = 0; i<phid->phid.attr.spatial.numGyroAxes; i++)
			{
				phid->gryoCorrection[i] = gryoCorrectionTemp[i] / (double)gryoCorrectionCount;
			}

			doneGyroZero = PTRUE;
		}
	}

	//see if it's time for an event
	if(timestampdiff(phid->latestDataTime, phid->lastEventTime) >= dataRate)
	{
		CPhidget_Timestamp tempTime;
		//int lastPtr;
		int counter = 0;
		int dataPerEvent = 0;

		int multipleDataPerEvent = PFALSE;

		if(dataRate < phid->interruptRate)
			multipleDataPerEvent = PTRUE;

		//max of 16 data per event
		eventData = malloc(16 * sizeof(CPhidgetSpatial_SpatialEventDataHandle));
		
		for(j=0;;j++)
		{
			//makes sure we read all data
			if(phid->bufferReadPtr == phid->bufferWritePtr || j>=16)
			{
				dataPerEvent = j;
				break;
			}

			eventData[j] = malloc(sizeof(CPhidgetSpatial_SpatialEventData));
			counter = 0;

			tempTime = phid->dataBuffer[phid->bufferReadPtr].timestamp;

			//average data for each stage
			while(phid->bufferReadPtr != phid->bufferWritePtr && 
				(!multipleDataPerEvent || timestampdiff(phid->dataBuffer[phid->bufferReadPtr].timestamp, tempTime) < dataRate))
			{
				for (i = 0; i<phid->phid.attr.spatial.numAccelAxes; i++)
				{
					if(phid->dataBuffer[phid->bufferReadPtr].acceleration[i] > phid->accelerationMax)
						phid->dataBuffer[phid->bufferReadPtr].acceleration[i] = phid->accelerationMax;
					if(phid->dataBuffer[phid->bufferReadPtr].acceleration[i] < phid->accelerationMin) 
						phid->dataBuffer[phid->bufferReadPtr].acceleration[i] = phid->accelerationMin;
					accelAvg[i] += phid->dataBuffer[phid->bufferReadPtr].acceleration[i];
				}
				for (i = 0; i<phid->phid.attr.spatial.numGyroAxes; i++)
				{
					double rate = phid->dataBuffer[phid->bufferReadPtr].angularRate[i] - phid->gryoCorrection[i];

					if(rate > phid->angularRateMax) 
						angularRateAvg[i] += phid->angularRateMax;
					else if(rate < phid->angularRateMin) 
						angularRateAvg[i] += phid->angularRateMin;
					else
						angularRateAvg[i] += rate;
				}
				for (i = 0; i<phid->phid.attr.spatial.numCompassAxes; i++)
				{
					if(phid->dataBuffer[phid->bufferReadPtr].magneticField[i] > phid->magneticFieldMax) 
						phid->dataBuffer[phid->bufferReadPtr].magneticField[i] = phid->magneticFieldMax;
					if(phid->dataBuffer[phid->bufferReadPtr].magneticField[i] < phid->magneticFieldMin) 
						phid->dataBuffer[phid->bufferReadPtr].magneticField[i] = phid->magneticFieldMin;
					magneticFieldAvg[i] += phid->dataBuffer[phid->bufferReadPtr].magneticField[i];
				}

				//lastPtr = phid->bufferReadPtr;

				phid->bufferReadPtr++;
				if(phid->bufferReadPtr >= SPATIAL_DATA_BUFFER_SIZE)
					phid->bufferReadPtr = 0;

				counter++;
			}

			for (i = 0; i<phid->phid.attr.spatial.numAccelAxes; i++)
			{
				eventData[j]->acceleration[i] = round_double(accelAvg[i] / (double)counter, 5);
				accelAvg[i] = 0;
			}
			for (i = 0; i<phid->phid.attr.spatial.numGyroAxes; i++)
			{
				if(phid->doZeroGyro && !doneGyroZero)
					eventData[j]->angularRate[i] = 0;
				else
					eventData[j]->angularRate[i] = round_double(angularRateAvg[i] / (double)counter, 5);
				angularRateAvg[i] = 0;
			}
			for (i = 0; i<phid->phid.attr.spatial.numCompassAxes; i++)
			{
				eventData[j]->magneticField[i] = round_double(magneticFieldAvg[i] / (double)counter, 5);
				magneticFieldAvg[i] = 0;
			}
			eventData[j]->timestamp = tempTime;
		}

		//store to local structure
		for( j = 0; j < dataPerEvent; j++)
		{
			for (i = 0; i<phid->phid.attr.spatial.numAccelAxes; i++)
			{
				accelAvg[i] += eventData[j]->acceleration[i];
			}
			for (i = 0; i<phid->phid.attr.spatial.numGyroAxes; i++)
			{
				angularRateAvg[i] += eventData[j]->angularRate[i];
			}
			for (i = 0; i<phid->phid.attr.spatial.numCompassAxes; i++)
			{
				magneticFieldAvg[i] += eventData[j]->magneticField[i];
			}
		}

		//Set local get data to averages
		for (i = 0; i<phid->phid.attr.spatial.numAccelAxes; i++)
		{
			phid->accelAxis[i] = round_double(accelAvg[i] / (double)dataPerEvent, 5);
		}
		for (i = 0; i<phid->phid.attr.spatial.numGyroAxes; i++)
		{
			if(phid->doZeroGyro && !doneGyroZero)
				phid->gyroAxis[i] = 0;
			else
				phid->gyroAxis[i] = round_double(angularRateAvg[i] / (double)dataPerEvent, 5);
		}
		for (i = 0; i<phid->phid.attr.spatial.numCompassAxes; i++)
		{
			phid->compassAxis[i] = round_double(magneticFieldAvg[i] / (double)dataPerEvent, 5);
		}
		
		//send out any events
		FIRE(SpatialData, eventData, dataPerEvent);

		phid->lastEventTime = phid->latestDataTime;

		for(i=0;i<dataPerEvent;i++)
			free(eventData[i]);
		free(eventData);
	}
done:

	//this will signal the zero function to return;
	if(doneGyroZero)
		phid->doZeroGyro = PFALSE;

	return EPHIDGET_OK;
}

//eventsAfterOpen - sends out an event for all valid data, used during attach initialization
CPHIDGETINITEVENTS(Spatial)
	TESTPTR(phid);
	//don't need to worry, because the interrupts come at a set rate
	return EPHIDGET_OK;
}

//getPacket - not used for spatial
CGETPACKET(Spatial)
	return EPHIDGET_UNEXPECTED;
}

// === Exported Functions === //

//create and initialize a device structure
CCREATE(Spatial, PHIDCLASS_SPATIAL)

//event setup functions
CFHANDLE(Spatial, SpatialData, CPhidgetSpatial_SpatialEventDataHandle *, int)

CGET(Spatial,AccelerationAxisCount,int)
	TESTPTRS(phid,pVal) 
	TESTDEVICETYPE(PHIDCLASS_SPATIAL)
	TESTATTACHED

	MASGN(phid.attr.spatial.numAccelAxes)
}
CGET(Spatial,GyroAxisCount,int)
	TESTPTRS(phid,pVal) 
	TESTDEVICETYPE(PHIDCLASS_SPATIAL)
	TESTATTACHED

	MASGN(phid.attr.spatial.numGyroAxes)
}
CGET(Spatial,CompassAxisCount,int)
	TESTPTRS(phid,pVal) 
	TESTDEVICETYPE(PHIDCLASS_SPATIAL)
	TESTATTACHED

	MASGN(phid.attr.spatial.numCompassAxes)
}

CGETINDEX(Spatial,Acceleration,double)
	TESTPTRS(phid,pVal) 	
	TESTDEVICETYPE(PHIDCLASS_SPATIAL)
	TESTATTACHED
	TESTINDEX(phid.attr.spatial.numAccelAxes)
	TESTMASGN(accelAxis[Index], PUNK_DBL)

	MASGN(accelAxis[Index])
}

CGETINDEX(Spatial,AccelerationMax,double)
	TESTPTRS(phid,pVal) 	
	TESTDEVICETYPE(PHIDCLASS_SPATIAL)
	TESTATTACHED
	TESTINDEX(phid.attr.spatial.numAccelAxes)
	TESTMASGN(accelerationMax, PUNK_DBL)

	MASGN(accelerationMax)
}

CGETINDEX(Spatial,AccelerationMin,double)
	TESTPTRS(phid,pVal) 	
	TESTDEVICETYPE(PHIDCLASS_SPATIAL)
	TESTATTACHED
	TESTINDEX(phid.attr.spatial.numAccelAxes)
	TESTMASGN(accelerationMin, PUNK_DBL)

	MASGN(accelerationMin)
}

CGETINDEX(Spatial,AngularRate,double)
	TESTPTRS(phid,pVal) 	
	TESTDEVICETYPE(PHIDCLASS_SPATIAL)
	TESTATTACHED
	TESTINDEX(phid.attr.spatial.numGyroAxes)
	TESTMASGN(gyroAxis[Index], PUNK_DBL)

	MASGN(gyroAxis[Index])
}

CGETINDEX(Spatial,AngularRateMax,double)
	TESTPTRS(phid,pVal) 	
	TESTDEVICETYPE(PHIDCLASS_SPATIAL)
	TESTATTACHED
	TESTINDEX(phid.attr.spatial.numGyroAxes)
	TESTMASGN(angularRateMax, PUNK_DBL)

	MASGN(angularRateMax)
}

CGETINDEX(Spatial,AngularRateMin,double)
	TESTPTRS(phid,pVal) 	
	TESTDEVICETYPE(PHIDCLASS_SPATIAL)
	TESTATTACHED
	TESTINDEX(phid.attr.spatial.numGyroAxes)
	TESTMASGN(angularRateMin, PUNK_DBL)

	MASGN(angularRateMin)
}

CGETINDEX(Spatial,MagneticField,double)
	TESTPTRS(phid,pVal) 	
	TESTDEVICETYPE(PHIDCLASS_SPATIAL)
	TESTATTACHED

	switch(phid->phid.deviceIDSpec)
	{
		/*
		case SUPPORTED_DEVICE:
			TESTINDEX(phid.attr.spatial.numCompassAxes)
			TESTMASGN(compassAxis[Index], PUNK_DBL)
			MASGN(compassAxis[Index])
			break;
			*/
		case PHIDID_ACCEL_3AXIS_GYRO_1AXIS:
		default:
			return EPHIDGET_UNSUPPORTED;
	}
	return EPHIDGET_OK;
}

CGETINDEX(Spatial,MagneticFieldMax,double)
	TESTPTRS(phid,pVal) 	
	TESTDEVICETYPE(PHIDCLASS_SPATIAL)
	TESTATTACHED

	switch(phid->phid.deviceIDSpec)
	{
		/*
		case SUPPORTED_DEVICE:
			TESTINDEX(phid.attr.spatial.numCompassAxes)
			TESTMASGN(magneticFieldMax, PUNK_DBL)
			MASGN(magneticFieldMax)
			break;
			*/
		case PHIDID_ACCEL_3AXIS_GYRO_1AXIS:
		default:
			return EPHIDGET_UNSUPPORTED;
	}
	return EPHIDGET_OK;
}

CGETINDEX(Spatial,MagneticFieldMin,double)
	TESTPTRS(phid,pVal) 	
	TESTDEVICETYPE(PHIDCLASS_SPATIAL)
	TESTATTACHED

	switch(phid->phid.deviceIDSpec)
	{
		/*
		case SUPPORTED_DEVICE:
			TESTINDEX(phid.attr.spatial.numCompassAxes)
			TESTMASGN(magneticFieldMin, PUNK_DBL)
			MASGN(magneticFieldMin)
			break;
			*/
		case PHIDID_ACCEL_3AXIS_GYRO_1AXIS:
		default:
			return EPHIDGET_UNSUPPORTED;
	}
	return EPHIDGET_OK;
}

CSET(Spatial,DataRate,int)
	TESTPTR(phid)
	TESTDEVICETYPE(PHIDCLASS_SPATIAL)
	TESTATTACHED
	TESTRANGE(phid->dataRateMax, phid->dataRateMin)

	if(CPhidget_statusFlagIsSet(phid->phid.status, PHIDGET_REMOTE_FLAG))
		;//TODO
	else
		phid->dataRate = newVal;

	return EPHIDGET_OK;
}
CGET(Spatial,DataRate,int)
	TESTPTRS(phid,pVal) 	
	TESTDEVICETYPE(PHIDCLASS_SPATIAL)
	TESTATTACHED
	TESTMASGN(dataRate, PUNK_INT)

	MASGN(dataRate)
}

CGET(Spatial,DataRateMax,int)
	TESTPTRS(phid,pVal) 	
	TESTDEVICETYPE(PHIDCLASS_SPATIAL)
	TESTATTACHED
	TESTMASGN(dataRateMax, PUNK_INT)

	MASGN(dataRateMax)
}

CGET(Spatial,DataRateMin,int)
	TESTPTRS(phid,pVal) 	
	TESTDEVICETYPE(PHIDCLASS_SPATIAL)
	TESTATTACHED
	TESTMASGN(dataRateMin, PUNK_INT)

	MASGN(dataRateMin)
}

PHIDGET21_API int CCONV CPhidgetSpatial_zeroGyro(CPhidgetSpatialHandle phid)
{
	TESTPTR(phid) 
	TESTDEVICETYPE(PHIDCLASS_SPATIAL)
	TESTATTACHED
	if(phid->phid.attr.spatial.numGyroAxes==0)
		return EPHIDGET_UNSUPPORTED;

	if(!phid->doZeroGyro)
	{
		phid->gyroZeroReadPtr = phid->bufferReadPtr;
		phid->doZeroGyro = PTRUE;
	}

	return EPHIDGET_OK;
}

//Maybe add these later
/*
CGET(Spatial,GyroHeading,double)
	TESTPTRS(phid,pVal) 	
	TESTDEVICETYPE(PHIDCLASS_SPATIAL)
	TESTATTACHED
	TESTMASGN(gyroHeading, PUNK_DBL)

	MASGN(gyroHeading)
}

CGET(Spatial,CompassHeading,double)
	TESTPTRS(phid,pVal) 	
	TESTDEVICETYPE(PHIDCLASS_SPATIAL)
	TESTATTACHED
	TESTMASGN(compassHeading, PUNK_DBL)

	MASGN(compassHeading)
}*/