/*********************************************************************
* 
*
* Instructions for Running:
*   1. Select the digital lines on the DAQ device to be written.
*   2. Select a value to write.
*   Note: The array is sized for 8 lines, if using a different
*     amount of lines, change the number of elements in the
*     array to equal the number of lines chosen.
*
* Steps:
*   1. Create a task.
*   2. Create a Digital Output channel. Use one channel for all
*        lines.
*   3. Call the Start function to start the task.
*   4. Write the digital Boolean array data. This write function
*        writes a single sample of digital data on demand, so no
*        timeout is necessary.
*   5. Call the Clear Task function to clear the Task.
*   6. Display an error if any.
*
*
*
*  Added to this version-0.0.1
*  While loop that returns the program to the scanf function to await another
*    command so that the next input is recognized. 
*
*  Added value of 3 to exit the while loop to enable user to exit the program
*
*  If the first command of the program does not pass a set of digital 
*    values to the card, daqmx throws an error about unsupported digital 
*    values.  This does not stop the program however, and it 
*    loops back to accept new input.  This can be avoided by commenting 
*    out lines 109 and 110 but you lose daqmx error *reporting.  
*    This should not impact the function of this program.
*
*********************************************************************/

#include <stdio.h>
#include <NIDAQmx.h>

#define DAQmxErrChk(functionCall) if( DAQmxFailed(error=(functionCall)) ) goto Error; else

int main(void)
{
  int         piocommand;
  int32       error=0;
  TaskHandle  taskHandle=0;
  uInt8       data[8];
  char        errBuff[2048]={'\0'};
  int	      exitflag ,a,b;

  exitflag = 0; 

  printf ("Please enter a command\n");
  printf ("0 = Line 0 high \n");
  printf ("1 = Line 1 high \n");
  printf ("2 = Line 2 high \n");
  printf ("3 = Line 3 high \n");
  printf ("4 = Line 4 high \n");
  printf ("5 = Line 5 high \n");
  printf ("6 = Line 6 high \n");
  printf ("7 = Line 7 high \n");
  printf ("8 = All lines low \n");
  printf ("9 = Exit the Dome Rotation Program\n");

  while ( !exitflag )
  {
    scanf( "%d", &piocommand );

    if (piocommand==0) 
    {
      data[0]= 1;
      data[1]= 0;
      data[2]= 0;
      data[3]= 0;
      data[4]= 0;
      data[5]= 0;
      data[6]= 0;
      data[7]= 0;
    }
    else if (piocommand==1)
    {
      data[0]= 0;
      data[1]= 1;
      data[2]= 0;
      data[3]= 0;
      data[4]= 0;
      data[5]= 0;
      data[6]= 0;
      data[7]= 0;
    } 
    else if (piocommand==2)
    {
      data[0]= 0;
      data[1]= 0;
      data[2]= 1;
      data[3]= 0;
      data[4]= 0;
      data[5]= 0;
      data[6]= 0;
      data[7]= 0;
    }    
    else if (piocommand==3) 
    {
      data[0]= 0;
      data[1]= 0;
      data[2]= 0;
      data[3]= 1;
      data[4]= 0;
      data[5]= 0;
      data[6]= 0;
      data[7]= 0;
    }
    else if (piocommand==4)
    {
      data[0]= 0;
      data[1]= 0;
      data[2]= 0;
      data[3]= 0;
      data[4]= 1;
      data[5]= 0;
      data[6]= 0;
      data[7]= 0;
    }        
    else  if (piocommand==5) 
    {
      data[0]= 0;
      data[1]= 0;
      data[2]= 0;
      data[3]= 0;
      data[4]= 0;
      data[5]= 1;
      data[6]= 0;
      data[7]= 0;
    }
    else if (piocommand==6)
    {
      data[0]= 0;
      data[1]= 0;
      data[2]= 0;
      data[3]= 0;
      data[4]= 0;
      data[5]= 0;
      data[6]= 1;
      data[7]= 0;
    }   
    else if (piocommand==7) 
    {
      data[0]= 0;
      data[1]= 0;
      data[2]= 0;
      data[3]= 0;
      data[4]= 0;
      data[5]= 0;
      data[6]= 0;
      data[7]= 1;
    }   
    else if (piocommand==9)
    {	
      exitflag = 1; 	
    }
    else 
    { 
      printf ("That is not a valid input.  Input 0,1,2 or 3.\n");
    }

    /*********************************************/
    // DAQmx Configure Code
    /*********************************************/
    
    /* This is where the device, line, and port are determined. */
    /* Port and lines depend on other cards installed and wiring. */
  
    DAQmxErrChk (DAQmxCreateTask("",&taskHandle));
    DAQmxErrChk (DAQmxCreateDOChan(taskHandle,"Dev1/port2/line0:7","",DAQmx_Val_ChanForAllLines));

    /*********************************************/
    // DAQmx Start Code
    /*********************************************/
    
    DAQmxErrChk (DAQmxStartTask(taskHandle));

    /*********************************************/
    // DAQmx Write Code
    /*********************************************/

    DAQmxErrChk (DAQmxWriteDigitalLines(taskHandle,1,1,10.0,DAQmx_Val_GroupByChannel,data,NULL,NULL));

    Error:
      if( DAQmxFailed(error) )
        DAQmxGetExtendedErrorInfo(errBuff,2048);
    
    if( taskHandle!=0 ) 
    {
      /*********************************************/
      // DAQmx Stop Code
      /*********************************************/
      
      DAQmxStopTask(taskHandle);
      DAQmxClearTask(taskHandle);
    }
    if( DAQmxFailed(error) )
      printf("DAQmx Error: %s\n",errBuff);
  }
  printf("Exiting\n");
  getchar();
  return 0;
}
