/* ----------------------------------------------------------------------- */
/* -                     Astronomical Dome Control                       - */
/* -                       RFID Dome Tag Capture                         - */
/* ----------------------------------------------------------------------- */
/*                                                                         */
/* Copyright (c) 2009 Jeff Hay and John Kielkopf                           */
/* kielkopf@louisville.edu                                                 */
/*                                                                         */
/* This file is part of XmDome                                             */
/*                                                                         */
/* XmDome is free software: you can redistribute it and/or modify          */
/* it under the terms of the GNU General Public License as published by    */
/* the Free Software Foundation, either version 3 of the License, or       */
/* (at your option) any later version.                                     */
/*                                                                         */
/* XmDome is distributed in the hope that it will be useful,               */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/* GNU General Public License for more details.                            */
/*                                                                         */
/* Credits: INDI components included here were derived from software       */
/* Copyright (C) 2003 Elwood C. Downey and originally distributed under    */
/* the terms of the GNU Lesser General Public License version 2.1.         */             
/*                                                                         */
/* Date: July 30, 2009                                                     */
/* Version: 1.1                                                            */



#include <stdio.h>
#include <phidget21.h>
#include <unistd.h>

int key;
int buf;
int losttag (CPhidgetRFIDHandle rfid, void *key, unsigned char *buf);
int gettag (CPhidgetRFIDHandle rfid, void *key, unsigned char *buf);

int main(int argc, const char * argv[]) 
{
  const char* type;	
  int dev; 	

  long status =0;

  CPhidgetRFIDHandle rfid = 0;
  CPhidgetRFID_create(&rfid);
  CPhidget_open((CPhidgetHandle)rfid,-1);

  CPhidget_waitForAttachment((CPhidgetHandle)rfid, 0);

  CPhidget_getDeviceVersion((CPhidgetHandle)rfid, &dev);
  CPhidget_getDeviceType((CPhidgetHandle)rfid, &type);

  printf("%10s, Tag Reader Device Version:  %d, \n", type, dev);

  CPhidgetRFID_setAntennaOn (rfid, 1);
  
  while (1)
  {
    CPhidgetRFID_set_OnTag_Handler(rfid, &gettag, NULL);
    CPhidgetRFID_set_OnTagLost_Handler(rfid, &losttag, NULL);
    sleep(1);
  }	

  return 0;
}

/* Handle case of tag moving in range */

int gettag (CPhidgetRFIDHandle rfid, void *key, unsigned char *buf)
{
  FILE* outfile;
  outfile = fopen("currenttag","w");
  fprintf(outfile, "%x%x%x%x%x%x%x%x%x%x\n", 
    buf[0]/16,buf[0]%16,buf[1]/16,buf[1]%16,buf[2]/16,buf[2]%16,
    buf[3]/16,buf[3]%16,buf[4]/16,buf[4]%16);
  fclose(outfile);
  return 0;
}

/* Handle case of tag moving out of range */

int losttag (CPhidgetRFIDHandle rfid, void *key, unsigned char *buf)
{
  
  /* printf("Tag out of range\n"); */
  
  return 0;
}
