/* ----------------------------------------------------------------------- */
/* -                     Astronomical Dome Control                       - */
/* -                       RFID Dome Tag Monitor                         - */
/* ----------------------------------------------------------------------- */
/*                                                                         */
/* Copyright (c) 2010 Jeff Hay and John Kielkopf                           */
/* kielkopf@louisville.edu                                                 */
/*                                                                         */
/* This file is part of XmDome                                             */
/*                                                                         */
/* XmDome is free software: you can redistribute it and/or modify          */
/* it under the terms of the GNU General Public License as published by    */
/* the Free Software Foundation, either version 3 of the License, or       */
/* (at your option) any later version.                                     */
/*                                                                         */
/* XmDome is distributed in the hope that it will be useful,               */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/* GNU General Public License for more details.                            */
/*                                                                         */
/* Credits: INDI components included here were derived from software       */
/* Copyright (C) 2003 Elwood C. Downey and originally distributed under    */
/* the terms of the GNU Lesser General Public License version 2.1.         */             
/*                                                                         */
/* Date: June 16, 2010                                                     */
/* Version: 1.3                                                            */

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdarg.h>
#include <stdlib.h>
#include <math.h>
#include <phidget21.h>
#include "monitordome.h"

/* Global parameters */

int key;
int ntags;
char dome_tag[MAXTAGS][20];
int dome_azimuth[MAXTAGS];
char current_tag[10];
int current_azimuth;

/* Pre-defined functions */

int losttag (CPhidgetRFIDHandle rfid, void *key, unsigned char *buf);
int gettag (CPhidgetRFIDHandle rfid, void *key, unsigned char *buf);
void read_config(void);

/* Files */

FILE *fp_config;                       /* Configuration file pointer */
static char *configfile;               /* Configuration file name */
void read_config(void);



int main(int argc, const char * argv[]) 
{
  const char* type;	
  int dev; 	

  long status =0;
  
  /* Read dome tag list with corresponding angles */
  
  read_config();

  /* Set up RFID tag monitor */
  
  CPhidgetRFIDHandle rfid = 0;
  CPhidgetRFID_create(&rfid);
  CPhidget_open((CPhidgetHandle)rfid,-1);

  CPhidget_waitForAttachment((CPhidgetHandle)rfid, 0);

  CPhidget_getDeviceVersion((CPhidgetHandle)rfid, &dev);
  CPhidget_getDeviceType((CPhidgetHandle)rfid, &type);

  /*
  printf("%10s, Tag Reader Device Version:  %d, \n", type, dev);
  */
  
  CPhidgetRFID_setAntennaOn (rfid, 1);
  
  /* Watch the dome continuously */
  
  while (1)
  {
    CPhidgetRFID_set_OnTag_Handler(rfid, &gettag, NULL);
    CPhidgetRFID_set_OnTagLost_Handler(rfid, &losttag, NULL);
    sleep(1);
  }	

  return 0;
}

/* Handle case of tag moving in range */

int gettag (CPhidgetRFIDHandle rfid, void *key, unsigned char *buf)
{
  int n;
  FILE* outfile;
  outfile = fopen("/usr/local/observatory/status/dometag","w");
  fprintf(outfile, "%x%x%x%x%x%x%x%x%x%x\n", 
    buf[0]/16,buf[0]%16,buf[1]/16,buf[1]%16,buf[2]/16,buf[2]%16,
    buf[3]/16,buf[3]%16,buf[4]/16,buf[4]%16);
  fclose(outfile);

  sprintf(current_tag, "%x%x%x%x%x%x%x%x%x%x\0", 
    buf[0]/16,buf[0]%16,buf[1]/16,buf[1]%16,buf[2]/16,buf[2]%16,
    buf[3]/16,buf[3]%16,buf[4]/16,buf[4]%16);
  
  /*
  printf("Current tag: %s\n",current_tag);
  */
  
   
  /* Search the list for a match */
  
  for(n=0; n<ntags; n++)
  {

    /*
    printf("Comp: %s %s \n",current_tag,dome_tag[n]);
    */
    
    if ( strstr(current_tag,dome_tag[n]) != NULL )
    {
      current_azimuth = dome_azimuth[n];
      outfile = fopen("/usr/local/observatory/status/domeazimuth","w");
      fprintf(outfile, "%d\n", current_azimuth);      
      fclose(outfile);
      
      /*
      printf("Current azimuth: %d\n", current_azimuth);     
      */
      
      break;
    }
  }   
  return 0;
}

/* Handle case of tag moving out of range */

int losttag (CPhidgetRFIDHandle rfid, void *key, unsigned char *buf)
{
  
  /* 
  printf("Tag out of range\n"); 
  */
  
  return 0;
}

/* Read and parse the initial configuration file */

void read_config(void)
{
  char configstr[121];
  char *configptr = configstr;
  int ntag=0;
  unsigned int azimuth;
    
  configfile = (char *) malloc (MAXPATHLEN);
  strcpy(configfile,CONFIGFILE);  
  fp_config = fopen(configfile, "r");
  
  if ( fp_config == NULL )
  {
    fprintf(stderr,"Caution: required dome configuration not found.\n");
    return;
  }
  else
  {
    
  }
  
  while ( configstr == fgets(configstr,80,fp_config) )
  {
    
    configptr = strstr(configstr,"dome.tag");
    if ( configptr != NULL)
    {
      configptr = strstr(configstr,"=");
      if ( configptr != NULL)
      {
        configptr = configptr + 1;
        sscanf(configptr,"%s %d", dome_tag[ntag], &dome_azimuth[ntag]);
        dome_tag[ntag][9]='\0';
        
        /*
        printf("%s %d\n", dome_tag[ntag], dome_azimuth[ntag]);
        */
        
        ntag++; 
        if (ntag >= MAXTAGS)
        {
          break;
        }    
      }  
    }
  }
  ntags = ntag;
  
  /*
  fprintf(stderr,"%d tags indexed.\n", ntags);  
  */
  
  if (ntag >= MAXTAGS)
  {
    fprintf(stderr,"Maximum number of tags allowed were found.");;
  }    
  fclose(fp_config);
}     
      
